NationWar = {}
NationWar.version = 1.3

NationWar.timer = Timer()
NationWar.abort = false

--defaults
NationWar.number = nil
NationWar.location = "Sedina D-2"
NationWar.starting_time = 20.5 --this is the hour in UTC; use decimals like .5 if you need to start at a :30 or whatever
NationWar.prize = "20 million"
NationWar.valk = nil

--channel number, or "T" for sector
NationWar.channel = 100

NationWar.maincolor = "\127FFFFFF"
NationWar.truecolor = "\12700FF00"
NationWar.falsecolor = "\127FF0000"
NationWar.othercolor = "\127FF8800"

--wrap the value in differeing colors based on whether it is true or false, and return the string.
--does not actually *print* anything, just returns the text
function NationWar.colorprint(val)
	if val then
		return(NationWar.truecolor.."TRUE")
	else
		return(NationWar.falsecolor.."FALSE")
	end
end

--print the settings
function NationWar.print()
	print(NationWar.maincolor .. "Number: " .. NationWar.othercolor .. tostring(NationWar.number))
	print(NationWar.maincolor .. "Location: " .. NationWar.othercolor .. tostring(NationWar.location))
	print(NationWar.maincolor .. "Start: " .. NationWar.othercolor .. tostring(NationWar.starting_time))
	print(NationWar.maincolor .. "Prize: " .. NationWar.othercolor .. tostring(NationWar.prize))
	print(NationWar.maincolor .. "Valk: " .. NationWar.colorprint(NationWar.valk))
	print(NationWar.maincolor .. "Channel: " .. NationWar.othercolor .. tostring(NationWar.channel))
end

--generate and return the text of the announcement
function NationWar.gen()
	local output = ""
	local seconds_to_go = NationWar.seconds_until_start()
	if (seconds_to_go > 0) then
		output = output .. NationWar.time_until_start(seconds_to_go) .. " until Nation War"
	else
		output = output .. "It is now time for Nation War"
	end
	if NationWar.number then
		output = output .. " #" .. NationWar.number
	end
	output = output .. " at " .. NationWar.location
	output = output .. ", " .. NationWar.prize .. " credit prize to the survivors!"
	if NationWar.valk ~= nil then
		if NationWar.valk then
			output = output .. "\nAll ships are allowed this week."
		else
			output = output .. "\nValkyries are not permitted this week."
		end
	end
	output = output .. "\nRemember:  Observers *MUST* use an EC-88 or EC-89."
	return(output)
end

--send "message" to "channel" at one line per second.  The user only uses those two parameters.  The "iteration" parameter is used internally when this function calls itself from a timer, to know how far through the message it is.
function NationWar.broadcast(message, channel, iteration)
	if (NationWar.abort) then
		NationWar.abort = false
		SendChat("Countdown ABORTED", "SECTOR")
		return
	end
	
	if (iteration == nil) then
		iteration = 1
	end
	
	local current = 1
	for line in message:gmatch("[^\r\n]+") do
		if (current == iteration) then
			if (channel == "T") then
				SendChat(line, "SECTOR")
			else
				SendChat(line, "CHANNEL", channel)
			end
			NationWar.timer:SetTimeout(1300,function() NationWar.broadcast(message, channel, iteration+1) end)
		end
		current = current + 1
	end;
end

--give a countdown in sector chat
function NationWar.countdown()
	local message = "5\n4\n3\n2\n1\nGO!!!"
	NationWar.broadcast(message, "T")
end

--print the rules - you should probably not do this on 100, it bothers some people
function NationWar.rules()
	local message = "\
Nation War is a 4-team last team standing battle, between the UIT, Serco, Itani, and Pirates (Grey).\
Pirates and outlaws may fight on the grey team.  Everybody else must be in their nation's respective team, regardless of faction standing.\
Everybody groups up with their team in " .. NationWar.location .. " before the event starts, distancing themselves from other teams.\
When it's time to start, the host will give a countdown, and then you can proceed with killing eachother.\
The last team standing wins, and the suriving members of that team split the prize money.\
During the fight, you have to stay within the asteroid field, and once you die, you're out.\
You can come back in a free bus (EC-88 or EC-89) to watch the rest of the fight, but you may NOT participate, and must stay far enough away from the action to not get in the way or distract.\
"
	if not NationWar.valk then
		message = message .. "\nThis week, you also are not allowed to use any Valkyrie model ships (this restriction only applies every other week)."
	end
	NationWar.broadcast(message, NationWar.channel)
end

--send the standard spiel about when NW starts
function NationWar.warning(channel)
	local message = tostring(NationWar.gen())
	NationWar.broadcast(message, NationWar.channel)
end

--demo the warning without actually sending it to other people, for proofreading and such
function NationWar.demo()
	local message = NationWar.gen()
	for line in message:gmatch("[^\r\n]+") do
		print(NationWar.maincolor .. line)
	end
end

--compute how many seconds are left before we start
function NationWar.seconds_until_start()
	local now = os.date("!*t")
	local start = os.date("!*t")
	start.hour = math.floor(NationWar.starting_time)
	start.min = math.floor((NationWar.starting_time - math.floor(NationWar.starting_time))*60)
	start.sec = 0
	return(os.date("%s",os.time(start)) - os.date("%s",os.time(now)))
end

--return a string saying how long in hours and minutes until we start
function NationWar.time_until_start(seconds_to_go)
	local output = ""
	local total_minutes_to_go = math.ceil(seconds_to_go/60)
	if (total_minutes_to_go > 90) then
		local hours_to_go = math.floor(total_minutes_to_go/60)
		local minutes_to_go = total_minutes_to_go - hours_to_go*60
		if (hours_to_go > 1) then
			output = output .. hours_to_go .. " hours"
		else
			output = output .. hours_to_go .. " hour"
		end
		if (hours_to_go > 0) and (minutes_to_go > 0) then
			output = output .. " and "
		end
		if (minutes_to_go > 1) then
			output = output .. minutes_to_go .. " minutes"
		elseif (minutes_to_go == 1) then
			output = output .. minutes_to_go .. " minute"
		end
	else
		if (total_minutes_to_go > 1) then
			output = output .. total_minutes_to_go .. " minutes"
		else
			output = output .. total_minutes_to_go .. " minute"
		end
	end
	return(output)
end

--Process commands
function NationWar.cmd(_,data)
	if data == nil then
		NationWar.warning()
	elseif data[1] == "number" then
		NationWar.number = data[2]
		NationWar.valk = (NationWar.number % 2 == 1)
		NationWar.print()
	elseif data[1] == "loc" then
		NationWar.location = data[2]
		NationWar.print()
	elseif data[1] == "start" then
		NationWar.starting_time = data[2]
		NationWar.print()
	elseif data[1] == "prize" then
		NationWar.prize = data[2]
		NationWar.print()
	elseif data[1] == "valk" then
		NationWar.valk = true
		NationWar.print()
	elseif data[1] == "novalk" then
		NationWar.valk = false
		NationWar.print()
	elseif data[1] == "chan" then
		NationWar.channel = data[2]
		NationWar.print()
	elseif data[1] == "go" then
		NationWar.countdown()
	elseif data[1] == "abort" then
		NationWar.abort = true
	elseif data[1] == "rules" then
		NationWar.rules()
	elseif data[1] == "warn" then
		NationWar.warning()
	elseif data[1] == "demo" then
		NationWar.demo()
	elseif data[1] == "print" then
		NationWar.print()
	else
		--help printout
		print(NationWar.maincolor .. "NationWar plugin")
		print(NationWar.maincolor .. "Options:")
		print(NationWar.maincolor .. "    number  |  loc  |  start  |  prize")
		print(NationWar.maincolor .. "    valk  |  novalk  |  go  |  abort")
		print(NationWar.maincolor .. "    chan  |  warn  |  demo  |  print")
		print(NationWar.maincolor .. "    rules")
	end
end
RegisterUserCommand("nationwar",NationWar.cmd)
RegisterUserCommand("nw",NationWar.cmd) --convenience
